@@ -12,22 +12,34 @@ from . import utils
 
 @utils.assert_logger
 def make_figure(height: Union[int, float] = 900,
-                width: Union[int, float] = 1300,
-                template: str = 'plotly_dark',
-                row_heights: List[Union[int, float]] = [10, 16, 7]) -> Figure:
+               width: Union[int, float] = 1300,
+               template: str = 'plotly_dark',
+               row_heights=None,
+               rows: int = 1,
+               cols: int = 1) -> Figure:
     assert isinstance(height, (int, float)), 'height must be of type <int> or <float>'
     assert isinstance(width, (int, float)), 'width must be of type <int> or <float>'
     assert isinstance(template, str), 'template must be of type <str>'
-    assert isinstance(row_heights, list), 'row_heights must be of type <List[int, float]>'
-    for el in row_heights:
-        assert isinstance(el, (int, float)), 'row_heights must be of type <List[int, float]>'
 
-    fig = make_subplots(3, 1, row_heights=row_heights)
+    fig = make_subplots(rows, cols, row_heights=row_heights)
     fig.update_layout(
         height=height,
         width=width,
         template=template,
         xaxis_rangeslider_visible=False)
+    return fig
+
+
+def make_trader_figure(height: Union[int, float] = 900,
+                      width: Union[int, float] = 1300,
+                      template: str = 'plotly_dark',
+                      row_heights: List[Union[int, float]] = [10, 16, 7]):
+    fig = make_figure(height=height,
+                    width=width,
+                    template=template,
+                    row_heights=row_heights,
+                    rows=3,
+                    cols=1)
     fig.update_xaxes(
         title_text=utils.TIME_TITLE, row=3, col=1, color=utils.TEXT_COLOR)
     fig.update_yaxes(
@@ -40,22 +52,11 @@ def make_figure(height: Union[int, float] = 900,
 
 
 class BaseGraph(object):
-    _figure: Figure
-    data_row: int = 1
-    data_col: int = 1
-    deposit_row: int = 2
-    deposit_col: int = 1
-    returns_row: int = 3
-    returns_col: int = 1
-
-    def __init__(self, figure: Figure):
+    def __init__(self, figure=None):
+        if figure is None:
+            figure = make_figure(rows=1, cols=1)
         self._figure = figure
 
-    def connect_trader(self, trader):
-        self.trader = trader
-        self.trader.fig = self
-        utils.logger.info('new %s graph', self.trader)
-
     def show(self, **kwargs):
         self._figure.show(**kwargs)
 
@@ -133,8 +134,31 @@ class BasePlotlyGraph(BaseGraph):
                        fill_color=utils.ICHIMOKU_CLOUD_COLOR,
                        opacity=utils.SENKOU_SPAN_B_ALPHA)
 
+    def log_y(self,
+              _row: int = 1,
+              _col: int = 1):
+        self._figure.update_yaxes(row=_row,
+                                 col=_col,
+                                 type='log')
 
-class QuickTradeGraph(BasePlotlyGraph):
+
+class TraderGraph(BasePlotlyGraph):
+    data_row: int = 1
+    data_col: int = 1
+    deposit_row: int = 2
+    deposit_col: int = 1
+    returns_row: int = 3
+    returns_col: int = 1
+
+    def __init__(self, figure=None):
+        if figure is None:
+            figure = make_figure(rows=3, cols=1)
+        self._figure = figure
+
+    def connect_trader(self, trader):
+        self.trader = trader
+        self.trader.fig = self
+        utils.logger.info('new %s graph', self.trader)
 
     def plot_deposit(self):
         deposit_start = self.trader.deposit_history[0]
@@ -164,7 +188,7 @@ class QuickTradeGraph(BasePlotlyGraph):
                        _col=self.returns_col)
 
     def plot_SL_TP_OPN(self):
-        self.plot_line(line=self.trader._stop_losses,
+        self.plot_line(line=self.trader.stop_losses,
                        width=utils.STOP_LOSS_WIDTH,
                        opacity=utils.STOP_LOSS_ALPHA,
                        color=utils.STOP_LOSS_COLOR,
@@ -172,7 +196,7 @@ class QuickTradeGraph(BasePlotlyGraph):
                        _row=self.data_row,
                        _col=self.data_col)
 
-        self.plot_line(line=self.trader._take_profits,
+        self.plot_line(line=self.trader.take_profits,
                        width=utils.TAKE_PROFIT_WIDTH,
                        opacity=utils.TAKE_PROFIT_ALPHA,
                        color=utils.TAKE_PROFIT_COLOR,
@@ -180,7 +204,7 @@ class QuickTradeGraph(BasePlotlyGraph):
                        _row=self.data_row,
                        _col=self.data_col)
 
-        self.plot_line(line=self.trader._open_lot_prices,
+        self.plot_line(line=self.trader.open_lot_prices,
                        width=utils.OPEN_TRADE_WIDTH,
                        opacity=utils.OPEN_TRADE_ALPHA,
                        color=utils.OPEN_TRADE_COLOR,
@@ -200,10 +224,10 @@ class QuickTradeGraph(BasePlotlyGraph):
         }
         for e, (pred, conv, crlev) in enumerate(zip(self.trader.returns,
                                                     self.trader._converted,
-                                                    utils.convert(self.trader._credit_leverages))):
+                                                    utils.convert(self.trader.credit_leverages))):
             if e != 0:
-                credlev_up = self.trader._credit_leverages[e - 1] < self.trader._credit_leverages[e]
-                credlev_down = self.trader._credit_leverages[e - 1] > self.trader._credit_leverages[e]
+                credlev_up = self.trader.credit_leverages[e - 1] < self.trader.credit_leverages[e]
+                credlev_down = self.trader.credit_leverages[e - 1] > self.trader.credit_leverages[e]
                 sell = (credlev_down and pred == utils.BUY) or (credlev_up and pred == utils.SELL)
                 buy = (credlev_down and pred == utils.SELL) or (credlev_up and pred == utils.BUY)
             else:
@@ -259,9 +283,39 @@ class QuickTradeGraph(BasePlotlyGraph):
                 width=width,
                 opacity=alpha)
 
-    def log_y(self,
-              _row: int = 1,
-              _col: int = 1):
-        self._figure.update_yaxes(row=_row,
-                                  col=_col,
-                                  type='log')
+
+class ValidationAnalysisGraph(BasePlotlyGraph):
+    test_row: int = 1
+    test_col: int = 1
+
+    def __init__(self, figure=None):
+        if figure is None:
+            figure = make_figure(rows=1, cols=1)
+        self._figure = figure
+
+    def connect_analyzer(self, analyzer):
+        self.analyzer = analyzer
+        self.analyzer.fig = self
+
+    def plot_frame(self):
+        if utils.STOP_BEFORE_INTEGER_AS_INDEX:
+            index = None
+        else:
+            index = self.analyzer.profit_keys
+        self.plot_line(line=self.analyzer.frame['train'],
+                       index=index,
+                       width=utils.STOP_BEFORE_TRAIN_WIDTH,
+                       opacity=utils.STOP_BEFORE_TRAIN_ALPHA,
+                       color=utils.STOP_BEFORE_TRAIN_COLOR,
+                       name=utils.STOP_BEFORE_TRAIN_NAME,
+                       _row=self.test_row,
+                       _col=self.test_col)
+
+        self.plot_line(line=self.analyzer.frame['validation'],
+                       index=index,
+                       width=utils.STOP_BEFORE_VAL_WIDTH,
+                       opacity=utils.STOP_BEFORE_VAL_ALPHA,
+                       color=utils.STOP_BEFORE_VAL_COLOR,
+                       name=utils.STOP_BEFORE_VAL_NAME,
+                       _row=self.test_row,
+                       _col=self.test_col)
